from visual import *

print """
Rotating coil in a magnetic field, operating as electric generator:
The green vectors denote the homogeneous magnetic field.
The current, induced in the rotating coil, is drawn in red,
the lorentz forces, acting on the four sides of the coil, in blue
and the magnetic flux, produced by the induced current, is drawn in blue-green color.

"""

grey=(0.85,0.85,0.85)

class slider:
    def __init__(self, pos=vector(0,0,0), axis=vector(0,1.,0), value=None,
                 width=0.15, min=0, max=100., color=(1,0,0)):
        pos = vector(pos)
       # axis = vector(axis)
        if value == None:
            value = min
        self.min = min
        self.max = max
        self.value = value
        self.shaft = cylinder(pos=pos, axis=axis, radius=width/4., color=grey)
        self.start = pos
        self.axis = axis
        self.control = sphere(pos=self.start+(self.min+value)/(self.max-self.min)*self.axis,
                        radius=width/2., color = color)
        self.label = label(pos=self.control.pos, text="%0.2f" % value,
                           opacity=0, box=0, line=0)

    def getslider(self, pos=None):
        pos = vector(pos)
        value = self.min+(self.max-self.min)*(pos.y-self.start.y)/mag(self.axis)
        self.setslider(value)
        return value

    def setslider(self, value=None):
        if value > self.max:
            value = self.max
        if value < self.min:
            value = self.min
        self.value = value
        self.control.pos = self.start+(self.min+value)/(self.max-self.min)*self.axis
        self.label.pos = self.control.pos
        self.label.text="%0.2f" % value

class Bfield:
    def __init__(self, pos=vector(0,0,0), axis=vector(.5,0,0),
                 width=5.0, nVec = 5, color=(0,1,0)):
        pos = vector(pos)
        axis = vector(axis)
        dGrid = width/nVec
        O_Grid = pos - vector(width/2,width/2,width/2)
        Bvecs = []
        for i in range(nVec+1):
            for j in range(nVec+1):
                for k in range(nVec+1):
                    arrowpos = O_Grid + dGrid*vector(i, j, k)
                    Bvecs.append( arrow( pos=vector(arrowpos), axis=axis, 
                                        color = color, shaftwidth = width/nVec/30) )

def centerdVect(vect, pos, axis):
      vect.pos=pos - axis/2.0
      vect.axis=axis

def setforcevectors(phi):
    force[0].pos=vector(sin(phi), -cos(phi),0 )*LSp/2
    force[0].axis=vector(0, sin(phi),0 )*LForce
    force[2].pos=vector(-sin(phi), cos(phi),0 )*LSp/2
    force[2].axis=vector(0, -sin(phi),0 )*LForce
    force[1].pos=vector(0, 0,1 )*LSp/2
    force[1].axis=-vector(0,0,1)*sin(phi)*cos(phi)*LForce
    force[3].pos=vector(0, 0,-1 )*LSp/2
    force[3].axis=-vector(0,0,-1)*sin(phi)*cos(phi)*LForce

    
scene.visible=0
scene.width = 600
scene.height = 600
scene.x = 10
scene.y = 10
scene.center = (0,0.5,0)
scene2 = display(title='Rotation Rate', width=300, height=600,
                center=(0,0.5,0), background=(0,1,1), x=scene.width+scene.x+5, y=scene.y)


scene.select()          # first window
scene.title = "Rotating Coil"

#draw B-field
Bfield1 = Bfield(pos=(0,0,0), axis=(.7,0,0), width=5.0, nVec = 5)

#Coil
LSp = 3.5
a=LSp/2
rdraht=LSp/50
Coil = curve(pos=[(0,-a,-0.9*a),(0,-a,0.9*a),(0,-0.97*a,0.97*a),
                   (0,-0.9*a,a),(0,0.9*a,a), (0,0.97*a,0.97*a),
                   (0,a,0.9*a),(0,a,-0.9*a),(0,0.97*a,-0.97*a),
                   (0,0.9*a,-a),(0,-0.9*a,-a),(0,-0.97*a,-0.97*a),
                   (0,-a,-0.9*a)])
Coil.radius=LSp/100
Coil.color=(1,0.7,0.2)


#Flux Area
fluxArea=box(pos=(0,0,0), length = rdraht/3, height=LSp-rdraht, width=LSp-rdraht, color=(0.5,0.5,0.5), opacity=0.7)

#Current Vector
LCurr=2
Currpos=Coil.pos[0] + (Coil.pos[1]-Coil.pos[0])/2.0
Curraxis0=vector(0,0,LSp)/LSp*LCurr
Curraxis=Curraxis0
Curr = arrow(color = (1,0,0), shaftwidth = Coil.radius*2)
centerdVect(vect=Curr, pos=Currpos, axis=Curraxis)

#B_self
B_self=arrow(pos = (0,0,0), axis=(2,0,0), color = (0,0.8,0.5), shaftwidth = Coil.radius*2)
B_selfaxis0=vector(-2,0,0)
B_selfaxis=B_selfaxis0

#Force Vectors
LForce0=1.3
LForce=LForce0
force=[]
for i in range(4):
  force.append(arrow(pos = Coil.pos[0] + (Coil.pos[1]-Coil.pos[0])/2, color = (0,0.5,1), shaftwidth = Coil.radius*2))
force[0].axis=(0,-LForce,0)
force[2].axis=(0,LForce,0)
force[1].axis=(0,0,LForce)
force[3].axis=(0,0,-LForce)

#frame
f=frame()
f.axis=(1,0,0)
f.pos=(0,0,0)
Coil.frame=f
Curr.frame=f
B_self.frame=f
fluxArea.frame=f
    
#scene2: Slider
scene2.select()
ctrl = slider(pos=(0,0,0), color=(1,0,0), min = 1., max=50., value=10.)
scene2.autoscale=1
omegalabel= label(pos=ctrl.start+ctrl.axis/2, text='omega', xoffset=0, yoffset=-10,
                    space=0.2, height=12, border=2)
scene2.autoscale=1
scene.visible=1
scene2.visible=1
scene.autoscale=0
scene2.autoscale=0
scene2.userzoom=0
scene2.userspin=0

scene.visible=1
phi=0.0
rotstep=-0.05
current_rate=ctrl.value
dragging=False
posi=vector(0,0,0)

while 1:
    rate(current_rate)
    f.rotate(angle=rotstep, axis=(0,0,1), origin=(0,0,0))
    phi=phi+rotstep
    centerdVect(vect=Curr, pos=Currpos, axis=Curraxis0*sin(phi)*current_rate/10)
    B_self.axis=B_selfaxis0*sin(phi)*current_rate/10
    setforcevectors(phi)
    if scene2.mouse.events:
        m = scene2.mouse.getevent()
        if m.drag=="left" and (m.pick is ctrl.control):
            posi = m.project(normal=(0,0,1))
            dragging=True
        if m.drop=="left":
            dragging=False
    newpos = scene2.mouse.project(normal=(0,0,1))
    if (newpos!= posi) and dragging:
        ctrl.getslider(posi)
        posi=newpos
        current_rate=ctrl.value
        LForce=LForce0*current_rate/10
